using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.OperationModel;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Threading.Tasks;

namespace TestOPC
{
    class Program
    {
        static readonly string node = "ns=3;s=\"OPC_Datenbaustein\".\"plcContainerID\"";

        private static void Output(UAAttributeData attributeDataResult)
        {
            if (attributeDataResult.HasGoodStatus)
                Console.WriteLine("AttributeData: {0}", attributeDataResult.Value);
            else
                Console.WriteLine("*** Failure: {0}", attributeDataResult.StatusCode);
        }

        static void Main(string[] args)
        {
            var sw = new Stopwatch();
            sw.Start();
            if (args[0] == "1")
                Main1();
            if (args[0] == "2")
                Main2();
            if (args[0] == "3")
                Main3();
            sw.Stop();
            Console.WriteLine($"Elapsed {sw.ElapsedMilliseconds}ms.");
        }

        public static void Main1()
        {
            EasyUAClientConfiguration config = new EasyUAClientConfiguration();
            config.SharedParameters.EngineParameters.CertificateAcceptancePolicy.AcceptAnyCertificate = true;
            var client = new EasyUAClient();

            UAAttributeDataResult[] attributeDataResultArray = client.ReadMultiple(new[]
                {
                    new UAReadArguments("opc.tcp://10.199.12.51:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.52:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.53:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.60:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.61:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.62:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.63:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.64:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.65:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.66:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.70:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.71:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.72:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.73:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.74:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.75:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.76:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.77:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.78:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.79:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.80:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.81:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.82:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.83:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.84:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.85:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.86:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.87:4840", node),
                    new UAReadArguments("opc.tcp://10.199.12.88:4840", node),
                });

            foreach (UAAttributeDataResult attributeDataResult in attributeDataResultArray)
            {
                if (attributeDataResult.Succeeded)
                    Console.WriteLine("AttributeData: {0}", attributeDataResult.AttributeData);
                else
                    Console.WriteLine("*** Failure: {0}", attributeDataResult.ErrorMessageBrief);
            }
        }

        public static void Main2()
        {
            EasyUAClientConfiguration config = new EasyUAClientConfiguration();
            config.SharedParameters.EngineParameters.CertificateAcceptancePolicy.AcceptAnyCertificate = true;
            var client = new EasyUAClient();

            Output(client.Read("opc.tcp://10.199.12.51:4840", node));
            Output(client.Read("opc.tcp://10.199.12.52:4840", node));
            Output(client.Read("opc.tcp://10.199.12.53:4840", node));
            Output(client.Read("opc.tcp://10.199.12.60:4840", node));
            Output(client.Read("opc.tcp://10.199.12.61:4840", node));
            Output(client.Read("opc.tcp://10.199.12.62:4840", node));
            Output(client.Read("opc.tcp://10.199.12.63:4840", node));
            Output(client.Read("opc.tcp://10.199.12.64:4840", node));
            Output(client.Read("opc.tcp://10.199.12.65:4840", node));
            Output(client.Read("opc.tcp://10.199.12.66:4840", node));
            Output(client.Read("opc.tcp://10.199.12.70:4840", node));
            Output(client.Read("opc.tcp://10.199.12.71:4840", node));
            Output(client.Read("opc.tcp://10.199.12.72:4840", node));
            Output(client.Read("opc.tcp://10.199.12.73:4840", node));
            Output(client.Read("opc.tcp://10.199.12.74:4840", node));
            Output(client.Read("opc.tcp://10.199.12.75:4840", node));
            Output(client.Read("opc.tcp://10.199.12.76:4840", node));
            Output(client.Read("opc.tcp://10.199.12.77:4840", node));
            Output(client.Read("opc.tcp://10.199.12.78:4840", node));
            Output(client.Read("opc.tcp://10.199.12.79:4840", node));
            Output(client.Read("opc.tcp://10.199.12.80:4840", node));
            Output(client.Read("opc.tcp://10.199.12.81:4840", node));
            Output(client.Read("opc.tcp://10.199.12.82:4840", node));
            Output(client.Read("opc.tcp://10.199.12.83:4840", node));
            Output(client.Read("opc.tcp://10.199.12.84:4840", node));
            Output(client.Read("opc.tcp://10.199.12.85:4840", node));
            Output(client.Read("opc.tcp://10.199.12.86:4840", node));
            Output(client.Read("opc.tcp://10.199.12.87:4840", node));
            Output(client.Read("opc.tcp://10.199.12.88:4840", node));
        }


        static void Main3()
        {
            EasyUAClientConfiguration config = new EasyUAClientConfiguration();
            config.SharedParameters.EngineParameters.CertificateAcceptancePolicy.AcceptAnyCertificate = true;
            var client = new EasyUAClient();

            var tasks = new List<Task>();
            tasks.Add(Task.Delay( 100).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.51:4840", node))));
            tasks.Add(Task.Delay( 200).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.52:4840", node))));
            tasks.Add(Task.Delay( 300).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.53:4840", node))));
            tasks.Add(Task.Delay( 400).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.60:4840", node))));
            tasks.Add(Task.Delay( 500).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.61:4840", node))));
            tasks.Add(Task.Delay( 600).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.62:4840", node))));
            tasks.Add(Task.Delay( 700).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.63:4840", node))));
            tasks.Add(Task.Delay( 800).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.64:4840", node))));
            tasks.Add(Task.Delay( 900).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.65:4840", node))));
            tasks.Add(Task.Delay(1000).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.66:4840", node))));
            tasks.Add(Task.Delay(1100).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.70:4840", node))));
            tasks.Add(Task.Delay(1200).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.71:4840", node))));
            tasks.Add(Task.Delay(1300).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.72:4840", node))));
            tasks.Add(Task.Delay(1400).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.73:4840", node))));
            tasks.Add(Task.Delay(1500).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.74:4840", node))));
            tasks.Add(Task.Delay(1600).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.75:4840", node))));
            tasks.Add(Task.Delay(1700).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.76:4840", node))));
            tasks.Add(Task.Delay(1800).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.77:4840", node))));
            tasks.Add(Task.Delay(1900).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.78:4840", node))));
            tasks.Add(Task.Delay(2100).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.79:4840", node))));
            tasks.Add(Task.Delay(2200).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.80:4840", node))));
            tasks.Add(Task.Delay(2300).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.81:4840", node))));
            tasks.Add(Task.Delay(2400).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.82:4840", node))));
            tasks.Add(Task.Delay(2500).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.83:4840", node))));
            tasks.Add(Task.Delay(2600).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.84:4840", node))));
            tasks.Add(Task.Delay(2700).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.85:4840", node))));
            tasks.Add(Task.Delay(2800).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.86:4840", node))));
            tasks.Add(Task.Delay(2900).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.87:4840", node))));
            tasks.Add(Task.Delay(3000).ContinueWith((t) => Output(client.Read("opc.tcp://10.199.12.88:4840", node))));
            Task.WaitAll(tasks.ToArray());                                               
        }
    }
}